# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Functional, browser based tests for radicale app.
"""

import logging

import pytest
import requests

from plinth.tests import functional

logger = logging.getLogger(__name__)

pytestmark = [pytest.mark.apps, pytest.mark.radicale]


class TestRadicaleApp(functional.BaseAppTests):
    app_name = 'radicale'
    has_service = True
    has_web = True

    def assert_app_running(self, session_browser):
        """Assert that the app is running."""
        assert functional.service_is_running(session_browser, self.app_name)
        assert _calendar_is_available(session_browser)
        assert _addressbook_is_available(session_browser)

    def assert_app_not_running(self, session_browser):
        """Assert that the app is not running."""
        assert functional.service_is_not_running(session_browser,
                                                 self.app_name)
        assert not _calendar_is_available(session_browser)
        assert not _addressbook_is_available(session_browser)

    def test_access_rights(self, session_browser):
        """Test setting the access rights."""
        functional.app_enable(session_browser, 'radicale')
        _set_access_rights(session_browser, 'owner_only')

        # Owner-write access rights
        _set_access_rights(session_browser, 'owner_write')
        assert functional.service_is_running(session_browser, 'radicale')
        assert _get_access_rights(session_browser) == 'owner_write'

        # Authenticated access rights
        _set_access_rights(session_browser, 'authenticated')
        assert functional.service_is_running(session_browser, 'radicale')
        assert _get_access_rights(session_browser) == 'authenticated'

        # Owner-only access rights
        _set_access_rights(session_browser, 'owner_only')
        assert functional.service_is_running(session_browser, 'radicale')
        assert _get_access_rights(session_browser) == 'owner_only'

    @pytest.mark.backups
    def test_backup_restore(self, session_browser):
        """Test backup and restore of configuration."""
        functional.app_enable(session_browser, 'radicale')
        _set_access_rights(session_browser, 'owner_only')
        functional.backup_create(session_browser, 'radicale', 'test_radicale')

        _set_access_rights(session_browser, 'owner_write')
        functional.backup_restore(session_browser, 'radicale', 'test_radicale')

        assert functional.service_is_running(session_browser, 'radicale')
        assert _get_access_rights(session_browser) == 'owner_only'


def _get_access_rights(browser):
    access_rights_types = ['owner_only', 'owner_write', 'authenticated']
    functional.nav_to_module(browser, 'radicale')
    for access_rights_type in access_rights_types:
        if browser.find_by_value(access_rights_type).checked:
            return access_rights_type


def _set_access_rights(browser, access_rights_type):
    functional.nav_to_module(browser, 'radicale')
    browser.choose('access_rights', access_rights_type)
    functional.submit(browser, form_class='form-configuration')


def _calendar_is_available(browser):
    """Return whether calendar is available at well-known URL."""
    conf = functional.config['DEFAULT']
    url = functional.base_url + '/.well-known/caldav'
    logging.captureWarnings(True)
    request = requests.get(url, auth=(conf['username'], conf['password']),
                           verify=False)
    logging.captureWarnings(False)
    return request.status_code != 404


def _addressbook_is_available(browser):
    """Return whether addressbook is available at well-known URL."""
    conf = functional.config['DEFAULT']
    url = functional.base_url + '/.well-known/carddav'
    logging.captureWarnings(True)
    request = requests.get(url, auth=(conf['username'], conf['password']),
                           verify=False)
    logging.captureWarnings(False)
    return request.status_code != 404



# SPDX-License-Identifier: AGPL-3.0-or-later
"""
FreedomBox app to configure RSS-Bridge.
"""
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from plinth import app as app_module
from plinth import cfg, frontpage, menu
from plinth.config import DropinConfigs
from plinth.modules.apache.components import Webserver
from plinth.modules.backups.components import BackupRestore
from plinth.modules.firewall.components import Firewall
from plinth.modules.users.components import UsersAndGroups
from plinth.package import Packages
from plinth.utils import format_lazy

from . import manifest, privileged

_description = [
    _('RSS-Bridge generates RSS and Atom feeds for websites that do not have '
      'one. Generated feeds can be consumed by any feed reader.'),
    format_lazy(
        _('When enabled, RSS-Bridge can be accessed by <a href="{users_url}">'
          'any user</a> belonging to the feed-reader group.'),
        users_url=reverse_lazy('users:index')),
    format_lazy(
        _('You can use RSS-Bridge with <a href="{ttrss_url}">Tiny Tiny '
          'RSS</a> to follow various websites. When adding a feed, enable '
          'authentication and use your {box_name} credentials.'),
        ttrss_url=reverse_lazy('ttrss:index'), box_name=cfg.box_name),
]


class RSSBridgeApp(app_module.App):
    """FreedomBox app for RSS-Bridge."""

    app_id = 'rssbridge'

    _version = 2

    def __init__(self) -> None:
        """Create components for the app."""
        super().__init__()

        groups = {'feed-reader': _('Read and subscribe to news feeds')}

        info = app_module.Info(app_id=self.app_id, version=self._version,
                               name=_('RSS-Bridge'), icon_filename='rssbridge',
                               short_description=_('RSS Feed Generator'),
                               description=_description,
                               manual_page='RSSBridge', donation_url=None,
                               clients=manifest.clients)
        self.add(info)

        menu_item = menu.Menu('menu-rssbridge', info.name,
                              info.short_description, info.icon_filename,
                              'rssbridge:index', parent_url_name='apps')
        self.add(menu_item)

        shortcut = frontpage.Shortcut('shortcut-rssbridge', name=info.name,
                                      short_description=info.short_description,
                                      icon=info.icon_filename,
                                      url='/rss-bridge/', login_required=True,
                                      allowed_groups=list(groups))
        self.add(shortcut)

        packages = Packages('packages-rssbridge', ['rss-bridge'])
        self.add(packages)

        dropin_configs = DropinConfigs('dropin-configs-rssbridge', [
            '/etc/apache2/conf-available/rss-bridge.conf',
        ])
        self.add(dropin_configs)

        firewall = Firewall('firewall-rssbridge', info.name,
                            ports=['http', 'https'], is_external=True)
        self.add(firewall)

        webserver = Webserver('webserver-rssbridge', 'rss-bridge',
                              urls=['https://{host}/rss-bridge/'])
        self.add(webserver)

        users_and_groups = UsersAndGroups('users-and-groups-rssbridge',
                                          groups=groups)
        self.add(users_and_groups)

        backup_restore = BackupRestore('backup-restore-rssbridge',
                                       **manifest.backup)
        self.add(backup_restore)

    def setup(self, old_version):
        """Install and configure the app."""
        super().setup(old_version)
        privileged.setup()
        if not old_version:
            self.enable()

    def uninstall(self):
        """De-configure and uninstall the app."""
        super().uninstall()
        privileged.uninstall()



# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Application manifest for bind.
"""

from django.utils.translation import gettext_lazy as _

from plinth.clients import store_url

clients = [{
    'name':
        _('Shaarlier'),
    'platforms': [{
        'type': 'store',
        'os': 'android',
        'store_name': 'google-play',
        'url': store_url('google-play', 'com.dimtion.shaarlier'),
    }, {
        'type': 'store',
        'os': 'android',
        'store_name': 'f-droid',
        'url': store_url('f-droid', 'com.dimtion.shaarlier'),
    }]
}, {
    'name': _('Shaarli'),
    'platforms': [{
        'type': 'web',
        'url': '/shaarli/'
    }]
}]

backup = {'data': {'directories': ['/var/lib/shaarli/data']}}



# SPDX-License-Identifier: AGPL-3.0-or-later
"""
Views for I2P application.
"""

from django.utils.translation import gettext_lazy as _

from plinth.views import AppView


class I2PAppView(AppView):
    """Serve configuration page."""
    app_id = 'i2p'
    template_name = 'i2p.html'
    proxies_description = [
        _('I2P lets you browse the Internet and hidden services (eepsites) '
          'anonymously. For this, your browser, preferably the Tor Browser, '
          'needs to be configured with a proxy.'),
        _('By default HTTP, HTTPS and IRC proxies are available. Additional '
          'proxies and tunnels may be configured using the tunnel '
          'configuration interface.'),
    ]
    torrents_description = [
        _('I2P provides an application to download files anonymously in a '
          'peer-to-peer network. Download files by adding torrents or '
          'create a new torrent to share a file.'),
    ]

    def get_context_data(self, **kwargs):
        """Return the context data for rendering the template view."""
        context = super().get_context_data(**kwargs)
        context['proxies_description'] = self.proxies_description
        context['torrents_description'] = self.torrents_description

        return context



# SPDX-License-Identifier: AGPL-3.0-or-later
"""Views for the minidlna module."""

import os

from django.contrib import messages
from django.utils.translation import gettext_lazy as _

from plinth.views import AppView

from . import privileged
from .forms import MiniDLNAServerForm


class MiniDLNAAppView(AppView):
    """Show minidlna app view."""

    app_id = 'minidlna'
    form_class = MiniDLNAServerForm

    def get_initial(self):
        """Return initial values of the form."""
        initial = super().get_initial()
        initial.update({'media_dir': privileged.get_media_dir()})
        return initial

    def form_valid(self, form):
        """Apply changes from the form."""
        old_config = form.initial
        new_config = form.cleaned_data

        if old_config['media_dir'].strip() != new_config['media_dir']:
            if os.path.isdir(new_config['media_dir']) is False:
                messages.error(self.request,
                               _('Specified directory does not exist.'))
            else:
                privileged.set_media_dir(new_config['media_dir'])
                messages.success(self.request, _('Updated media directory'))

        return super().form_valid(form)
